<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\Shape\Chart\Type\AbstractType;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Area;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Bar;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Line;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Pie;
use PhpOffice\PhpPresentation\Shape\Chart\Type\Scatter;

class ChartBox extends FloatingObject
{
    public $type = 'chart_box';

    protected function setContent()
    {
        $chart = new Node();
        $chart->type = 'chart';

        $config = new \stdClass();

        $title = new \stdClass();
        $title->text = $this->phpShape->getTitle()->getText();
        $config->title = $title;

        $plotArea = new \stdClass();
        $plotArea->switchRowColumn = true;
        $plotArea->xAxis = new \stdClass();
        $plotArea->yAxis = new \stdClass();
        $plotArea->series = array();
        $plotArea->blankAs = "zero";

        $phpPlotArea = $this->phpShape->getPlotArea();
        $plotArea->xAxis->title = new \stdClass();
        $plotArea->xAxis->title->text = $phpPlotArea->getAxisX()->getTitle();
        $plotArea->yAxis->title = new \stdClass();
        $plotArea->yAxis->title->text = $phpPlotArea->getAxisY()->getTitle();

        $phpTypes = $phpPlotArea->getTypes();
        $seriesIndex = 0;
        $hybrid = false;
        foreach ($phpTypes as $phpType) {
            $type = null;

            if ($phpType instanceof Area) {
                $type = 'area';
            } else if ($phpType instanceof Bar) {
                $type = 'bar';
            } else if ($phpType instanceof Line) {
                $type = 'line';
            } else if ($phpType instanceof Pie) {
                $type = 'pie';
            } else if ($phpType instanceof Scatter) {
                $type = 'scatter';
            } else {
                continue;
            }
            if ($hybrid === false) {
                $hybrid = $type;
            } else if ($hybrid !== $type) {
                $hybrid = true;
            }

            if (AbstractType::DIRECTION_VERTICAL === $phpType->getDirection()) {
                $config->barDir = 'col';
            } else {
                $config->barDir = 'row';
            }
            $config->grouping = (AbstractType::GROUPING_STACKED === $phpType->getGrouping()) ? 'stacked' : 'standard';

            foreach ($phpType->getSeries() as $phpSeries) {
                $series = new \stdClass();
                $series->type = $type;

                $marker = new \stdClass();
                $marker->symbol = $phpType->isMarker() ? 'circle' : 'none';
                $marker->size = 5;
                $series->marker = $marker;

                // Fill corner data and cat.
                if (!$data) {
                    $data = array();
                    $data[0] = array();
                    $data[0][0] = '';
                    foreach (array_keys($phpSeries->getValues()) as $key => $cat) {
                        $data[$key + 1] = array();
                        $data[$key + 1][0] = $cat;
                    }
                }

                // Fill tx and val.
                $dataIndex = $seriesIndex + 1;
                $data[0][$dataIndex] = $phpSeries->getTitle();
                foreach (array_values($phpSeries->getValues()) as $key => $val) {
                    if (!$data[$key + 1]) {
                        $data[$key + 1] = array();
                    }
                    $data[$key + 1][$dataIndex] = $val;
                }

                $plotArea->series[] = $series;
                $seriesIndex++;
            }
        }
        $config->chartType = ($hybrid === true) ? 'hybrid' : $plotArea->series[0]->type;
        $config->plotArea = $plotArea;

        $chart->setAttr('data', $data);
        $chart->setAttr('config', $config);
        $this->addContent($chart);
    }
}


?>
