const { Transform } = require('stream');

class Base64Decoder extends Transform {
	constructor() {
		super();
		this._reminder = null;
	}

	_transform(chunk, encoding, callback) {
		let part = this._reminder ? Buffer.concat([this._reminder, chunk]) : chunk;
		const remaining = part.length % 4;
		if (remaining) {
			this._reminder = part.slice(part.length - remaining);
			part = part.slice(0, part.length - remaining);
		} else {
			this._reminder = null;
		}
		this.push(decodeBuffer(part));
		callback();
	}

	_flush(callback) {
		if (this._reminder) {
			this.push(decodeBuffer(this._reminder));
		}
		callback();
	}
}

class Base64Encoder extends Transform {
	constructor() {
		super();
		this._reminder = null;
	}

	_transform(chunk, encoding, callback) {
		let part = this._reminder ? Buffer.concat([this._reminder, chunk]) : chunk;
		const remaining = part.length % 3;
		if (remaining) {
			this._reminder = part.slice(part.length - remaining);
			part = part.slice(0, part.length - remaining);
		} else {
			this._reminder = null;
		}
		this.push(encodeBuffer(part));
		callback();
	}

	_flush(callback) {
		if (this._reminder) {
			this.push(encodeBuffer(this._reminder));
		}
		callback();
	}
}

function decodeBuffer(b64Buffer) {
	return Buffer.from(b64Buffer.toString(), 'base64');
}

function encodeBuffer(buf) {
	return Buffer.from(buf.toString('base64'));
}

module.exports = {
	Base64Encoder,
	Base64Decoder,
};
